@@ -182,7 +182,9 @@ class AlgebraicMixin(ParameterMixin, CompoundMixin, BaseModel):
                 "out": set(v.derived_compounds),
             }
 
-        available_compounds = set(self.compounds) | set(self.parameters) | {"time"}
+        available_compounds = (
+            set(self.compounds) | self.get_all_parameter_names() | {"time"}
+        )
         module_order = []
         modules_to_sort: SimpleQueue = SimpleQueue()
         for k, v in self.algebraic_modules.items():
@@ -216,15 +218,15 @@ class AlgebraicMixin(ParameterMixin, CompoundMixin, BaseModel):
     ) -> None:
         self._check_for_existence(
             name=module.name,
-            check_against=self.get_all_compounds(),
-            candidates=module.compounds + module.modifiers,
+            check_against=set(self.get_all_compounds()),
+            candidates=set(module.compounds + module.modifiers),
             element_type="compound",
             raise_on_missing=False,
         )
         self._check_for_existence(
             name=module.name,
-            check_against=self.get_parameter_names(),
-            candidates=module.parameters,
+            check_against=set(self.get_parameter_names()) | set(self.derived_parameters),
+            candidates=set(module.parameters),
             element_type="parameter",
         )
         if check_ids:
@@ -322,13 +324,13 @@ class AlgebraicMixin(ParameterMixin, CompoundMixin, BaseModel):
         sort_modules: bool = True,
         **meta_info: Dict[str, Any],
     ) -> None:
-
         compounds = []
         modifiers = []
         parameters = []
 
+        param_names = self.get_all_parameter_names()
         for i in args:
-            if i in self.parameters:
+            if i in param_names:
                 parameters.append(i)
             elif i in self.compounds:
                 compounds.append(i)
@@ -454,8 +456,11 @@ class AlgebraicMixin(ParameterMixin, CompoundMixin, BaseModel):
             compounds = []
             modifiers = []
             parameters = []
+
+            param_names = self.get_all_parameter_names()
+
             for i in args:
-                if i in self.parameters:
+                if i in param_names:
                     parameters.append(i)
                 elif i in self.compounds:
                     compounds.append(i)
@@ -577,7 +582,7 @@ class AlgebraicMixin(ParameterMixin, CompoundMixin, BaseModel):
         """
         y["time"] = t
         # args = self.parameters | y # that is python 3.9+ syntax, wait for a bit
-        args: Dict[str, Union[float, Array]] = {**self.parameters, **y}
+        args: Dict[str, Union[float, Array]] = {**self.get_all_parameters(), **y}
         for name in self._algebraic_module_order:
             module = self.algebraic_modules[name]
             try:
