@@ -102,8 +102,8 @@ class BaseModel(ABC):
         self,
         *,
         name: str,
-        check_against: List[str],
-        candidates: List[str],
+        check_against: set[str],
+        candidates: set[str],
         element_type: str,
         raise_on_missing: bool = False,
     ) -> None:
