@@ -9,6 +9,7 @@ import json
 import pickle
 import warnings
 from dataclasses import dataclass, field
+from queue import Empty, SimpleQueue
 from typing import Any, Callable, Dict, Iterable, List, Optional
 
 import libsbml
@@ -34,17 +35,33 @@ class Parameter:
     notes: dict = field(default_factory=dict)
 
 
+@dataclass
+class DerivedParameter:
+    function: Callable
+    parameters: list[str]
+
+    def __getitem__(self, key: str) -> Any:
+        return self.__dict__[key]
+
+
 class ParameterMixin(BaseModel):
     """Adding parameter functions."""
 
     def __init__(self, parameters: dict[str, float] | None = None) -> None:
-        self.parameters: Dict[str, float] = {}
-        self.derived_parameters: Dict[str, Dict[str, Any]] = {}
+        self.derived_parameters: dict[str, DerivedParameter] = {}
+        self._parameters: dict[str, float] = {}
+        self._derived_parameter_order: list[str] = []
+
+        # Bookkeeping
         self._derived_from_parameters: set[str] = set()
 
         if parameters is not None:
             self.add_parameters(parameters=parameters)
-        self.initialization_parameters = self.parameters.copy()
+        self.initialization_parameters = self._parameters.copy()
+
+    @property
+    def parameters(self) -> dict[str, float]:
+        return self.get_all_parameters()
 
     ##########################################################################
     # Meta info
@@ -72,7 +89,11 @@ class ParameterMixin(BaseModel):
     ##########################################################################
 
     def add_parameter(
-        self, parameter_name: str, parameter_value: float, **meta_info: Dict[str, Any]
+        self,
+        parameter_name: str,
+        parameter_value: float,
+        update_derived: bool = True,
+        **meta_info: Dict[str, Any],
     ) -> None:
         """Add a new parameter to the model.
 
@@ -83,9 +104,10 @@ class ParameterMixin(BaseModel):
             {unit, database_links, notes}
         """
         self._check_and_insert_ids([parameter_name], context="add_parameter")
-        self.add_and_update_parameter(parameter_name, parameter_value)
         self._add_parameter_meta_info(parameter_name, meta_info)
 
+        self._parameters[parameter_name] = parameter_value
+
     def add_parameters(
         self,
         parameters: Dict[str, float],
@@ -105,6 +127,7 @@ class ParameterMixin(BaseModel):
         self,
         parameter_name: str,
         parameter_value: float,
+        update_derived: bool = True,
         **meta_info: Dict[str, Any],
     ) -> None:
         """Update an existing model parameter.
@@ -114,13 +137,15 @@ class ParameterMixin(BaseModel):
         UserWarning
             If parameter is not in the model
         """
-        if parameter_name not in self.parameters:
+        if parameter_name not in self._parameters:
+            if parameter_name in self._derived_from_parameters:
+                raise ValueError(f"{parameter_name} is a derived parameter")
             warnings.warn(f"Key {parameter_name} is not in the model. Adding.")
-        self.add_and_update_parameter(parameter_name, parameter_value)
         self.update_parameter_meta_info(parameter_name, meta_info)
+        self._parameters[parameter_name] = parameter_value
 
     def scale_parameter(self, parameter_name: str, factor: float) -> None:
-        self.parameters[parameter_name] *= factor
+        self.update_parameter(parameter_name, self._parameters[parameter_name] * factor)
 
     def update_parameters(
         self,
@@ -151,16 +176,13 @@ class ParameterMixin(BaseModel):
     ) -> None:
         """Add a new or update an existing parameter."""
         if parameter_name not in self._ids:
-            self._check_and_insert_ids(
-                [parameter_name], context="add_and_update_parameter"
+            self.add_parameter(
+                parameter_name, parameter_value, update_derived, **meta_info
             )
-            self._add_parameter_meta_info(parameter_name, meta_info)
         else:
-            self.update_parameter_meta_info(parameter_name, meta_info)
-
-        self.parameters[parameter_name] = parameter_value
-        if parameter_name in self._derived_from_parameters and update_derived:
-            self._update_derived_parameters()
+            self.update_parameter(
+                parameter_name, parameter_value, update_derived, **meta_info
+            )
 
     def add_and_update_parameters(
         self,
@@ -184,7 +206,7 @@ class ParameterMixin(BaseModel):
 
     def remove_parameter(self, parameter_name: str) -> None:
         """Remove a parameter from the model."""
-        del self.parameters[parameter_name]
+        del self._parameters[parameter_name]
         if parameter_name in self.meta_info:
             del self.meta_info["parameters"][parameter_name]
         self._remove_ids([parameter_name])
@@ -203,6 +225,48 @@ class ParameterMixin(BaseModel):
     # Derived parameter functions
     ##########################################################################
 
+    def _sort_derived_parameters(self, max_iterations: int = 10_000) -> None:
+        available = set(self.get_parameter_names())
+        par_order = []
+        pars_to_sort: SimpleQueue = SimpleQueue()
+        for k, v in self.derived_parameters.items():
+            pars_to_sort.put((k, set(v.parameters)))
+
+        last_name = None
+        i = 0
+        name: str
+        args: set[str]
+        while True:
+            try:
+                name, args = pars_to_sort.get_nowait()
+            except Empty:
+                break
+            if args.issubset(available):
+                available.add(name)
+                par_order.append(name)
+            else:
+                if last_name == name:
+                    par_order.append(name)
+                    break
+                pars_to_sort.put((name, args))
+                last_name = name
+            i += 1
+            if i > max_iterations:
+                to_sort = []
+                while True:
+                    try:
+                        to_sort.append(pars_to_sort.get_nowait()[0])
+                    except Empty:
+                        break
+
+                raise ValueError(
+                    "Exceeded max iterations on derived parameter module sorting. "
+                    "Check if there are circular references.\n"
+                    f"Available: {to_sort}\n"
+                    f"Order: {par_order}"
+                )
+        self._derived_parameter_order = par_order
+
     def add_derived_parameter(
         self,
         parameter_name: str,
@@ -218,21 +282,24 @@ class ParameterMixin(BaseModel):
         # Do this first to check if all parameters are actually in the model
         # Since we also need the values to compute anything, don't use _check_for_existence here
         # that would be an unnecessary duplicate effort
-        parameter_values = [self.parameters[i] for i in parameters]
+        # parameter_values = [self._parameters[i] for i in parameters]
 
-        self.derived_parameters[parameter_name] = {
-            "function": function,
-            "parameters": parameters,
-        }
+        self.derived_parameters[parameter_name] = DerivedParameter(
+            function=function,
+            parameters=parameters,
+        )
         for parameter in parameters:
             self._derived_from_parameters.add(parameter)
 
         # Initial calculation
-        self.add_parameter(
-            parameter_name=parameter_name,
-            parameter_value=function(*parameter_values),
-            **meta_info,
-        )
+        # self.add_parameter(
+        #     parameter_name=parameter_name,
+        #     parameter_value=function(*parameter_values),
+        #     update_derived=True,
+        #     **meta_info,
+        # )
+
+        self._sort_derived_parameters()
 
     def update_derived_parameter(
         self,
@@ -243,14 +310,14 @@ class ParameterMixin(BaseModel):
     ) -> None:
         old = self.derived_parameters[parameter_name]
         if function is None:
-            function = old["function"]
+            function = old.function
         if parameters is None:
-            parameters = old["parameters"]
+            parameters = old.parameters
 
-        self.derived_parameters[parameter_name]["function"] = function
-        self.derived_parameters[parameter_name]["parameters"] = parameters
-        self._update_derived_parameters()
+        self.derived_parameters[parameter_name].function = function
+        self.derived_parameters[parameter_name].parameters = parameters
         self.update_parameter_meta_info(parameter_name, meta_info)
+        self._sort_derived_parameters()
 
     def remove_derived_parameter(self, parameter_name: str) -> None:
         """Remove a derived parameter from the model."""
@@ -259,25 +326,7 @@ class ParameterMixin(BaseModel):
         for i in derived_from:
             if all(i not in j["parameters"] for j in self.derived_parameters.values()):
                 self._derived_from_parameters.remove(i)
-        self.remove_parameter(parameter_name=parameter_name)
-
-    def _update_derived_parameters(self) -> None:
-        """Update values of all derived parameters.
-
-        This function is supposed to be run after a change to any parameter
-        from which a derived parameter is calculated. Since this operation
-        is assumed to take not a lot of time, it is run for all parameters.
-        If there ever is a model with lots of derived parameters, it might
-        make sense to make this a little less brute-force ;)
-        """
-        for parameter_name, parameter in self.derived_parameters.items():
-            self.add_and_update_parameter(
-                parameter_name=parameter_name,
-                parameter_value=parameter["function"](
-                    *(self.parameters[i] for i in parameter["parameters"])
-                ),
-                update_derived=False,
-            )
+        self._sort_derived_parameters()
 
     def store_parameters_to_file(
         self,
@@ -289,12 +338,12 @@ class ParameterMixin(BaseModel):
             if not filename.endswith(".json"):
                 filename += ".json"
             with open(filename, "w") as f:
-                json.dump(self.parameters, f)
+                json.dump(self._parameters, f)
         elif filetype == "pickle":
             if not filename.endswith(".p"):
                 filename += ".p"
             with open(filename, "wb") as f:  # type: ignore
-                pickle.dump(self.parameters, f)  # type: ignore
+                pickle.dump(self._parameters, f)  # type: ignore
         else:
             raise ValueError("Can only save to json or pickle")
 
@@ -315,19 +364,34 @@ class ParameterMixin(BaseModel):
 
     def restore_initialization_parameters(self) -> None:
         """Restore parameters to initialization parameters."""
-        self.parameters = self.initialization_parameters.copy()
+        self._parameters = self.initialization_parameters.copy()
 
     def get_parameter(self, parameter_name: str) -> float:
         """Return the value of a single parameter."""
-        return float(self.parameters[parameter_name])
+        return float(self._parameters[parameter_name])
 
     def get_parameters(self) -> Dict[str, float]:
         """Return all parameters."""
-        return dict(self.parameters)
+        return dict(self._parameters)
+
+    def get_all_parameters(self) -> dict[str, float]:
+        all_params = self._parameters.copy()
+
+        for parameter_name in self._derived_parameter_order:
+            derived_parameter = self.derived_parameters[parameter_name]
+            all_params[parameter_name] = derived_parameter.function(
+                *(all_params[i] for i in derived_parameter.parameters)
+            )
+
+        return all_params
 
     def get_parameter_names(self) -> List[str]:
         """Return names of all parameters"""
-        return list(self.parameters)
+        return list(self._parameters)
+
+    def get_all_parameter_names(self) -> set[str]:
+        """Return names of all parameters"""
+        return set(self._parameters) | set(self.derived_parameters)
 
     ##########################################################################
     # Source code functions
@@ -349,9 +413,7 @@ class ParameterMixin(BaseModel):
         parameter_modelbase_code : str
             Source code generating the modelbase parameters
         """
-        parameters = repr(
-            {k: v for k, v in self.parameters.items() if k not in self.derived_parameters}
-        )
+        parameters = repr({k: v for k, v in self._parameters.items()})
         # derived_pars =
         if include_meta_info:
             meta_info = self._get_nonzero_meta_info(component="parameters")
@@ -416,7 +478,7 @@ class ParameterMixin(BaseModel):
         ----------
         sbml_model : libsbml.Model
         """
-        for parameter_id, value in self.parameters.items():
+        for parameter_id, value in self._parameters.items():
             parameter = self.meta_info["parameters"][parameter_id]
             k = sbml_model.createParameter()
             k.setId(convert_id_to_sbml(id_=parameter_id, prefix="PAR"))
