@@ -209,14 +209,14 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
     def _check_rate_consistency(self, rate: Rate) -> None:
         self._check_for_existence(
             name=rate.name,
-            check_against=self._get_all_compounds(),
-            candidates=rate.dynamic_variables,
+            check_against=set(self._get_all_compounds()),
+            candidates=set(rate.dynamic_variables),
             element_type="compound",
         )
         self._check_for_existence(
             name=rate.name,
-            check_against=self.get_parameter_names(),
-            candidates=rate.parameters,
+            check_against=set(self.get_parameter_names()) | set(self.derived_parameters),
+            candidates=set(rate.parameters),
             element_type="parameter",
         )
         rate.check_consistency()
@@ -451,7 +451,11 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
             self._check_rate_consistency(rate)
         self.update_rate_meta_info(rate_name, meta_info)
 
-    def update_rates(self, rates: dict, meta_info: Optional[Dict[str, Any]] = None,) -> None:
+    def update_rates(
+        self,
+        rates: dict,
+        meta_info: Optional[Dict[str, Any]] = None,
+    ) -> None:
         """Update multiple rates."""
         meta_info = {} if meta_info is None else meta_info
         for rate_name, rate in rates.items():
@@ -521,7 +525,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
         fcd: Dict[str, float],
     ) -> Dict[str, float]:
         # args = self.parameters | fcd # python 3.9+ syntax
-        args = {**self.parameters, **fcd}
+        args = {**self.get_all_parameters(), **fcd}
         fluxes = {}
         for name, rate in self.rates.items():
             try:
@@ -536,7 +540,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
         fcd: Dict[str, float],
     ) -> Dict[str, Array]:
         # args = self.parameters | fcd # python 3.9+ syntax
-        args = {**self.parameters, **fcd}
+        args = {**self.get_all_parameters(), **fcd}
         fluxes = {}
         for name, rate in self.rates.items():
             try:
