@@ -962,7 +962,6 @@ class LabelModel(_AbstractRateModel, BaseModel):
         >>> get_full_concentration_dict(y=[0, 0])
         >>> get_full_concentration_dict(y={"X": 0, "Y": 0})
         """
-        self._update_derived_parameters()
         if isinstance(t, (int, float)):
             t = [t]
         t = np.array(t)
@@ -984,7 +983,6 @@ class LabelModel(_AbstractRateModel, BaseModel):
         t: Union[float, ArrayLike, Array] = 0.0,
     ) -> Dict[str, Array]:
         """Calculate the fluxes at time point(s) t."""
-        self._update_derived_parameters()
         fcd = self.get_full_concentration_dict(y=y, t=t)  # type: ignore
         ones = np.ones(len(fcd["time"]))
         if len(fcd["time"]) == 1:
@@ -1136,7 +1134,7 @@ class LabelModel(_AbstractRateModel, BaseModel):
         from . import Model
 
         m = Model()
-        m.add_parameters(self.parameters)
+        m.add_parameters(self.get_parameters())
         m.add_compounds(list(self.label_compounds.keys()))
         m.add_compounds(self.nonlabel_compounds)
 
