@@ -1,6 +1,7 @@
 """The main class for modeling. Provides model construction and inspection tools."""
 
 from __future__ import annotations
+
 from dataclasses import dataclass
 
 __all__ = [
@@ -126,7 +127,7 @@ class Model(_AbstractRateModel, BaseModel):
     def __iadd__(self, other: "Model") -> "Model":
         self.add(**{"compounds": self._element_difference(other, "compounds")})  # type: ignore
 
-        for k, v in other.parameters.items():
+        for k, v in other.get_parameters().items():
             if k not in other.derived_parameters:
                 self.add_and_update_parameter(k, v)
 
@@ -145,6 +146,14 @@ class Model(_AbstractRateModel, BaseModel):
             else:
                 self.add_algebraic_module(name, **dict(module))
 
+        for rate_name, cpd_dict in other.derived_stoichiometries.items():
+            for cpd_name, der_stoich in cpd_dict.items():
+                self._add_derived_stoichiometry(
+                    rate_name=rate_name,
+                    cpd_name=cpd_name,
+                    derived_stoichiometry=der_stoich,
+                )
+
         for attribute in [
             "rates",
             "stoichiometries",
@@ -163,6 +172,7 @@ class Model(_AbstractRateModel, BaseModel):
             "rates",
             "stoichiometries",
             "functions",
+            "derived_stoichiometries",
         ]:
             m.remove(**{attribute: self._element_intersection(other, attribute)})  # type: ignore
         return m
@@ -170,6 +180,19 @@ class Model(_AbstractRateModel, BaseModel):
     def __isub__(self, other: "Model") -> Model:
         return self.copy().__sub__(other)
 
+    def _collect_used_parameters(self) -> set[str]:
+        used_parameters = set()
+        for par in self.derived_parameters.values():
+            used_parameters.update(par["parameters"])
+        for module in self.algebraic_modules.values():
+            used_parameters.update(module.parameters)
+        for rate in self.rates.values():
+            used_parameters.update(rate.parameters)
+        for cpd_dict in self.derived_stoichiometries.values():
+            for der_stoich in cpd_dict.values():
+                used_parameters.update(der_stoich.args)
+        return used_parameters
+
     def add(
         self,
         compounds: list[str] | None = None,
@@ -219,6 +242,7 @@ class Model(_AbstractRateModel, BaseModel):
         rates: List[str] | None = None,
         stoichiometries: List[str] | None = None,
         functions: List[str] | None = None,
+        derived_stoichiometries: list[str] | None = None,
     ) -> None:
         if compounds is not None:
             self.remove_compounds(compounds)
@@ -232,6 +256,8 @@ class Model(_AbstractRateModel, BaseModel):
             self.remove_rate_stoichiometries(stoichiometries)
         if functions is not None:
             self.remove_functions(functions)
+        if derived_stoichiometries is not None:
+            self._remove_derived_stoichiometries(derived_stoichiometries)
 
     ##########################################################################
     # Reactions
@@ -246,24 +272,37 @@ class Model(_AbstractRateModel, BaseModel):
         self._update_derived_stoichiometries()
 
     def _update_derived_stoichiometries(self) -> None:
+        parameters = self.get_all_parameters()
         for rate_name, d in self.derived_stoichiometries.items():
             for cpd_name, stoichiometry in d.items():
                 new_stoichiometry = stoichiometry.function(
-                    *(self.parameters[i] for i in stoichiometry.args)
+                    *(parameters[i] for i in stoichiometry.args)
                 )
 
-                self.stoichiometries.setdefault(rate_name, {}).update({cpd_name: new_stoichiometry})
+                self.stoichiometries.setdefault(rate_name, {}).update(
+                    {cpd_name: new_stoichiometry}
+                )
                 self.stoichiometries_by_compounds.setdefault(cpd_name, {}).update(
                     {rate_name: new_stoichiometry}
                 )
 
+    def _remove_derived_stoichiometry(self, rate_name: str) -> None:
+        self.derived_stoichiometries.pop(rate_name, None)
+
+    def _remove_derived_stoichiometries(self, rate_names: list[str]) -> None:
+        for rate_name in rate_names:
+            self._remove_derived_stoichiometry(rate_name)
+
     def update_parameter(
         self,
         parameter_name: str,
         parameter_value: float,
+        update_derived: bool = True,
         **meta_info: Dict[str, Any],
     ) -> None:
-        super().update_parameter(parameter_name, parameter_value, **meta_info)
+        super().update_parameter(
+            parameter_name, parameter_value, update_derived, **meta_info
+        )
         self._update_derived_stoichiometries()
 
     def update_parameters(
@@ -362,8 +401,10 @@ class Model(_AbstractRateModel, BaseModel):
         modifiers = []
         parameters = []
         dynamic_variables = []
+
+        par_names = self.get_all_parameter_names()
         for i in args:
-            if i in self.parameters:
+            if i in par_names:
                 parameters.append(i)
             elif i not in stoichiometry:
                 modifiers.append(i)
@@ -475,8 +516,10 @@ class Model(_AbstractRateModel, BaseModel):
             modifiers = []
             parameters = []
             dynamic_variables = []
+
+            par_names = self.get_all_parameter_names()
             for i in args:
-                if i in self.parameters:
+                if i in par_names:
                     parameters.append(i)
                 elif i not in stoichiometry:
                     modifiers.append(i)
@@ -563,6 +606,7 @@ class Model(_AbstractRateModel, BaseModel):
         """
         self.remove_rate(rate_name=rate_name)
         self.remove_rate_stoichiometry(rate_name=rate_name)
+        self._remove_derived_stoichiometry(rate_name=rate_name)
 
     def remove_reactions(self, rate_names: List[str]) -> None:
         """Remove multiple reactions from the model.
@@ -615,7 +659,6 @@ class Model(_AbstractRateModel, BaseModel):
         >>> get_full_concentration_dict(y=[0, 0])
         >>> get_full_concentration_dict(y={"X": 0, "Y": 0})
         """
-        self._update_derived_parameters()
         if isinstance(t, (int, float)):
             t = [t]
         t = np.array(t)
@@ -637,7 +680,6 @@ class Model(_AbstractRateModel, BaseModel):
         t: Union[float, ArrayLike, Array] = 0.0,
     ) -> Dict[str, Array]:
         """Calculate the fluxes at time point(s) t."""
-        self._update_derived_parameters()
         fcd = self.get_full_concentration_dict(y=y, t=t)  # type: ignore
         ones = np.ones(len(fcd["time"]))
         if len(fcd["time"]) == 1:
@@ -691,7 +733,7 @@ class Model(_AbstractRateModel, BaseModel):
         from modelbase.ode import LabelModel
 
         lm = LabelModel()
-        lm.add_parameters(self.parameters)
+        lm.add_parameters(self.get_parameters())
         for compound in self.get_compounds():
             if compound in labelcompounds:
                 lm.add_label_compound(
