@@ -11,11 +11,10 @@ import pickle
 import sys
 import warnings
 from abc import ABC, abstractmethod
-from concurrent import futures
+from concurrent.futures import ProcessPoolExecutor
 from functools import partial
 from typing import (
     Any,
-    Callable,
     Dict,
     Generic,
     Iterable,
@@ -32,7 +31,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 from tqdm.auto import tqdm
-from typing_extensions import Literal
+from typing_extensions import Literal, Self
 
 from ...typing import Array, ArrayLike, Axes, Axis, Figure
 from ...utils.plotting import _get_plot_kwargs, _style_subplot, plot, plot_grid
@@ -216,10 +215,26 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
         )
         return time_array, results_array
 
+    def simulate_and(
+        self,
+        t_end: Optional[float] = None,
+        steps: Optional[int] = None,
+        time_points: Optional[ArrayLike] = None,
+        **integrator_kwargs: Dict[str, Any],
+    ) -> Self:
+        self.simulate(
+            t_end=t_end,
+            steps=steps,
+            time_points=time_points,
+            **integrator_kwargs,
+        )
+        return self
+
     def simulate_to_steady_state(
         self,
         tolerance: float = 1e-6,
         simulation_kwargs: Dict[str, Any] | None = None,
+        rel_norm: bool = False,
         **integrator_kwargs: Dict[str, Any],
     ) -> Tuple[Optional[Array], Optional[Array]]:
         """Simulate the model."""
@@ -231,6 +246,7 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
             tolerance=tolerance,
             simulation_kwargs=simulation_kwargs,
             integrator_kwargs=integrator_kwargs,
+            rel_norm=rel_norm,
         )
         if time is None or results is None:
             return None, None
@@ -241,6 +257,21 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
         )
         return time_array, results_array
 
+    def simulate_to_steady_state_and(
+        self,
+        tolerance: float = 1e-6,
+        simulation_kwargs: Dict[str, Any] | None = None,
+        rel_norm: bool = False,
+        **integrator_kwargs: Dict[str, Any],
+    ) -> Self:
+        self.simulate_to_steady_state(
+            tolerance=tolerance,
+            simulation_kwargs=simulation_kwargs,
+            rel_norm=rel_norm,
+            **integrator_kwargs,
+        )
+        return self
+
     @overload
     def get_results_array(  # type: ignore
         self,
@@ -482,6 +513,10 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         new = copy.deepcopy(self)
         if self.simulation_parameters is not None:
             new.simulation_parameters = self.simulation_parameters.copy()
+        if self.fluxes is not None:
+            new.fluxes = self.fluxes.copy()
+        if self.full_results is not None:
+            new.full_results = self.full_results.copy()
         if new.results is not None:
             new._initialise_integrator(y0=new.results[-1])
         elif new.y0 is not None:
@@ -509,7 +544,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         self,
         y0: Union[ArrayLike, Dict[str, float]],
         test_run: bool = True,
-    ) -> None:
+    ) -> Self:
         """Initialise the integrator."""
         if self.results is not None:
             self.clear_results()
@@ -521,6 +556,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
 
         if test_run:
             self._test_run()
+        return self
 
     def update_parameter(
         self,
@@ -533,17 +569,31 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             parameter_name=parameter_name, parameter_value=parameter_value
         )
 
+    def update_parameter_and(
+        self,
+        parameter_name: str,
+        parameter_value: float,
+    ) -> Self:
+        self.update_parameter(
+            parameter_name=parameter_name, parameter_value=parameter_value
+        )
+        return self
+
     def update_parameters(self, parameters: Dict[str, float]) -> None:
         """Update model parameters."""
         self.model.update_parameters(parameters=parameters)
 
+    def update_parameters_and(self, parameters: Dict[str, float]) -> Self:
+        self.update_parameters(parameters=parameters)
+        return self
+
     def _save_simulation_results(
         self, *, time: Array, results: Array, skipfirst: bool
     ) -> None:
         super()._save_simulation_results(time=time, results=results, skipfirst=skipfirst)
         if self.simulation_parameters is None:
             self.simulation_parameters = []
-        self.simulation_parameters.append(self.model.parameters.copy())
+        self.simulation_parameters.append(self.model.get_parameters())
 
     def simulate(
         self,
@@ -568,7 +618,6 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         integrator_kwargs : dict
             Integrator options
         """
-        self.model._update_derived_parameters()
         time, results = super().simulate(
             t_end=t_end,
             steps=steps,
@@ -583,13 +632,14 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         self,
         tolerance: float = 1e-8,
         simulation_kwargs: Dict[str, Any] | None = None,
+        rel_norm: bool = False,
         **integrator_kwargs: Dict[str, Any],
     ) -> Tuple[Optional[Array], Optional[Array]]:
         """Simulate the model to steady state."""
-        self.model._update_derived_parameters()
         time, results = super().simulate_to_steady_state(
             tolerance=tolerance,
             simulation_kwargs=simulation_kwargs,
+            rel_norm=rel_norm,
             **integrator_kwargs,
         )
         self.full_results = None
@@ -1005,6 +1055,21 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         full_results_df = cast(List[pd.DataFrame], full_results_df)
         return [i.loc[:, variables].values for i in full_results_df]
 
+    def get_right_hand_side(
+        self,
+        *,
+        annotate_names: bool = True,
+    ) -> pd.DataFrame:
+        rhs = pd.DataFrame()
+        for t, y, p in zip(self.time, self.results, self.simulation_parameters):  # type: ignore
+            self.update_parameters(p)
+            _rhs = [
+                self.model.get_right_hand_side(y=yi, t=ti, annotate_names=annotate_names)
+                for ti, yi in zip(t, y)
+            ]
+            rhs = pd.concat((rhs, pd.DataFrame(_rhs, index=t)))
+        return rhs
+
     @staticmethod
     def _parameter_scan_worker(
         parameter_value: float,
@@ -1014,15 +1079,18 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         Sim: Type[_BaseRateSimulator],
         integrator: Type[AbstractIntegrator],
         tolerance: float,
-        y0: Dict[str, float],
+        y0: list[float],
         integrator_kwargs: Dict[str, Any],
         include_fluxes: bool,
+        rel_norm: bool,
     ) -> Tuple[float, Dict[str, float], Dict[str, float]]:
         m = model.copy()
         s = Sim(model=m, integrator=integrator)
         s.initialise(y0=y0, test_run=False)
         s.update_parameter(parameter_name=parameter_name, parameter_value=parameter_value)
-        t, y = s.simulate_to_steady_state(tolerance=tolerance, **integrator_kwargs)
+        t, y = s.simulate_to_steady_state(
+            tolerance=tolerance, rel_norm=rel_norm, **integrator_kwargs
+        )
         if t is None or y is None:
             concentrations = dict(
                 zip(
@@ -1044,65 +1112,28 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         concentrations = dict(s.get_full_results_df(concatenated=True).iloc[-1])  # type: ignore
         return parameter_value, concentrations, fluxes  # type: ignore
 
-    def _scan_threading(
-        self, parameter_name: str, parameter_values: ArrayLike, worker: Callable
-    ) -> list:
-        with tqdm(total=len(parameter_values), desc=parameter_name) as pbar:
-            results = []
-            for value in parameter_values:
-                results.append(worker(value))
-                pbar.update(1)
-        return results
-
-    def _scan_multiprocessing(
-        self, parameter_name: str, parameter_values: ArrayLike, worker: Callable
-    ) -> list:
-        with tqdm(total=len(parameter_values), desc=parameter_name) as pbar:
-            with futures.ProcessPoolExecutor() as executor:
-                results = []
-                for task in futures.as_completed(
-                    (executor.submit(worker, i) for i in parameter_values)
-                ):
-                    results.append(task.result())
-                    pbar.update(1)
-        return results
-
     def parameter_scan(
         self,
         parameter_name: str,
         parameter_values: ArrayLike,
         tolerance: float = 1e-8,
         multiprocessing: bool = True,
+        max_workers: int | None = None,
+        disable_tqdm: bool = False,
+        rel_norm: bool = False,
         **integrator_kwargs: Dict[str, Any],
     ) -> pd.DataFrame:
         """Scan the model steady state changes caused by a change to a parameter."""
-        if sys.platform in ["win32", "cygwin"]:
-            warnings.warn(
-                """
-                Windows does not behave well with multiple processes.
-                Falling back to threading routine."""
-            )
-        worker = partial(
-            self._parameter_scan_worker,
-            **{
-                "parameter_name": parameter_name,
-                "model": self.model,
-                "Sim": self.__class__,
-                "integrator": self._integrator,
-                "tolerance": tolerance,
-                "y0": self.y0,
-                "integrator_kwargs": integrator_kwargs,
-                "include_fluxes": False,
-            },
-        )
-        if sys.platform in ["win32", "cygwin"] or not multiprocessing:
-            results = self._scan_threading(parameter_name, parameter_values, worker)
-        else:
-            results = self._scan_multiprocessing(parameter_name, parameter_values, worker)
-        concentrations = {}
-        for (i, conc, _) in results:
-            concentrations[i] = conc
-        return pd.DataFrame(concentrations).T.sort_index()
+        return self.parameter_scan_with_fluxes(
+            parameter_name=parameter_name,
+            parameter_values=parameter_values,
+            tolerance=tolerance,
+            multiprocessing=multiprocessing,
+            max_workers=max_workers,
+            disable_tqdm=disable_tqdm,
+            rel_norm=rel_norm,
+            **integrator_kwargs,
+        )[0]
 
     def parameter_scan_with_fluxes(
         self,
@@ -1110,6 +1141,9 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         parameter_values: ArrayLike,
         tolerance: float = 1e-8,
         multiprocessing: bool = True,
+        disable_tqdm: bool = False,
+        max_workers: int | None = None,
+        rel_norm: bool = False,
         **integrator_kwargs: Dict[str, Any],
     ) -> Tuple[pd.DataFrame, pd.DataFrame]:
         """Scan the model steady state changes caused by a change to a parameter."""
@@ -1121,29 +1155,36 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             )
         worker = partial(
             self._parameter_scan_worker,
-            **{
-                "parameter_name": parameter_name,
-                "model": self.model,
-                "Sim": self.__class__,
-                "integrator": self._integrator,
-                "tolerance": tolerance,
-                "y0": self.y0,
-                "integrator_kwargs": integrator_kwargs,
-                "include_fluxes": True,
-            },
+            parameter_name=parameter_name,
+            model=self.model,
+            Sim=self.__class__,
+            integrator=self._integrator,
+            tolerance=tolerance,
+            y0=self.y0,
+            integrator_kwargs=integrator_kwargs,
+            include_fluxes=True,
+            rel_norm=rel_norm,
         )
+
+        results: Iterable[tuple[float, dict[str, Array], dict[str, Array]]]
         if sys.platform in ["win32", "cygwin"] or not multiprocessing:
-            results = self._scan_threading(parameter_name, parameter_values, worker)
+            results = tqdm(
+                map(worker, parameter_values),
+                total=len(parameter_values),
+                desc=parameter_name,
+                disable=disable_tqdm,
+            )
         else:
-            results = self._scan_multiprocessing(parameter_name, parameter_values, worker)
+            with ProcessPoolExecutor(max_workers=max_workers) as pe:
+                results = tqdm(pe.map(worker, parameter_values))
         concentrations = {}
         fluxes = {}
-        for (i, conc, flux) in results:
+        for i, conc, flux in results:
             concentrations[i] = conc
             fluxes[i] = flux
         return (
-            pd.DataFrame(concentrations).T.sort_index(),
-            pd.DataFrame(fluxes).T.sort_index(),
+            pd.DataFrame(concentrations).T,
+            pd.DataFrame(fluxes).T,
         )
 
     def parameter_scan_2d(
@@ -1151,13 +1192,16 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         p1: Tuple[str, ArrayLike],
         p2: Tuple[str, ArrayLike],
         tolerance: float = 1e-8,
+        disable_tqdm: bool = False,
         multiprocessing: bool = True,
+        max_workers: int | None = None,
+        rel_norm: bool = False,
         **integrator_kwargs: Dict[str, Any],
     ) -> Dict[float, pd.DataFrame]:
         cs = {}
         parameter_name1, parameter_values1 = p1
         parameter_name2, parameter_values2 = p2
-        original_pars = self.model.parameters.copy()
+        original_pars = self.model.get_parameters().copy()
         for value in tqdm(
             parameter_values2, total=len(parameter_values2), desc=parameter_name2
         ):
@@ -1166,7 +1210,10 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
                 parameter_name1,
                 parameter_values1,
                 tolerance=tolerance,
+                disable_tqdm=disable_tqdm,
                 multiprocessing=multiprocessing,
+                max_workers=max_workers,
+                rel_norm=rel_norm,
                 **integrator_kwargs,
             )
         self.update_parameters(original_pars)
@@ -1177,14 +1224,17 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         p1: Tuple[str, ArrayLike],
         p2: Tuple[str, ArrayLike],
         tolerance: float = 1e-8,
+        disable_tqdm: bool = False,
         multiprocessing: bool = True,
+        max_workers: int | None = None,
+        rel_norm: bool = False,
         **integrator_kwargs: Dict[str, Any],
     ) -> Tuple[Dict[float, pd.DataFrame], Dict[float, pd.DataFrame]]:
         cs = {}
         vs = {}
         parameter_name1, parameter_values1 = p1
         parameter_name2, parameter_values2 = p2
-        original_pars = self.model.parameters.copy()
+        original_pars = self.model.get_parameters().copy()
         for value in tqdm(
             parameter_values2, total=len(parameter_values2), desc=parameter_name2
         ):
@@ -1194,7 +1244,10 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
                 parameter_values1,
                 tolerance=tolerance,
                 multiprocessing=multiprocessing,
-                **integrator_kwargs,
+                disable_tqdm=disable_tqdm,
+                max_workers=max_workers,
+                rel_norm=rel_norm,
+                **integrator_kwargs,  # type: ignore
             )
             cs[value] = c
             vs[value] = v
@@ -1433,8 +1486,8 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         ncols: int | None = None,
         sharex: bool = True,
         sharey: bool = True,
-        xlabels: List[str] | None = None,
-        ylabels: List[str] | None = None,
+        xlabels: str | Iterable[str] | None = None,
+        ylabels: str | Iterable[str] | None = None,
         normalise: Union[float, ArrayLike] | None = None,
         plot_titles: Optional[Iterable[str]] = None,
         figure_title: str | None = None,
@@ -1460,7 +1513,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         )
         if y is None:
             return None, None
-        plot_groups = [(y.loc[:, compounds].values,) for compounds in compound_groups]
+        plot_groups = [(y.loc[:, compounds],) for compounds in compound_groups]
         return plot_grid(
             plot_groups=plot_groups,  # type: ignore
             legend_groups=compound_groups,
@@ -1483,6 +1536,47 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             title_kwargs=title_kwargs,
         )
 
+    def plot_derivatives(
+        self,
+        compounds: list[str],
+        xlabel: str | None = None,
+        ylabel: str | None = None,
+        title: str | None = None,
+        grid: bool = True,
+        tight_layout: bool = True,
+        ax: Optional[Axis] = None,
+        figure_kwargs: Dict[str, Any] | None = None,
+        subplot_kwargs: Dict[str, Any] | None = None,
+        plot_kwargs: Dict[str, Any] | None = None,
+        grid_kwargs: Dict[str, Any] | None = None,
+        legend_kwargs: Dict[str, Any] | None = None,
+        tick_kwargs: Dict[str, Any] | None = None,
+        label_kwargs: Dict[str, Any] | None = None,
+        title_kwargs: Dict[str, Any] | None = None,
+    ) -> tuple[Figure | None, Axis | None]:
+        rhs = self.get_right_hand_side(annotate_names=False)
+        if len(rhs) == 0:
+            return None, None
+
+        return plot(
+            plot_args=(rhs.loc[:, compounds],),
+            legend=compounds,
+            xlabel=xlabel,
+            ylabel=ylabel,
+            title=title,
+            grid=grid,
+            tight_layout=tight_layout,
+            ax=ax,
+            figure_kwargs=figure_kwargs,
+            subplot_kwargs=subplot_kwargs,
+            plot_kwargs=plot_kwargs,
+            grid_kwargs=grid_kwargs,
+            legend_kwargs=legend_kwargs,
+            tick_kwargs=tick_kwargs,
+            label_kwargs=label_kwargs,
+            title_kwargs=title_kwargs,
+        )
+
     def plot_against_variable(
         self,
         variable: str,
@@ -1779,9 +1873,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         if fluxes is None:
             return None, None
         fluxes = cast(pd.DataFrame, fluxes)
-        plot_groups = [
-            (cast(Array, fluxes.loc[:, group].values),) for group in rate_groups
-        ]
+        plot_groups = [(cast(Array, fluxes.loc[:, group]),) for group in rate_groups]
         return plot_grid(
             plot_groups=plot_groups,  # type: ignore
             legend_groups=rate_groups,
@@ -2159,3 +2251,25 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         if tight_layout:
             fig.tight_layout()
         return fig, ax
+
+    def plot_production_and_consumption(
+        self, cpd: str
+    ) -> tuple[Optional[Figure], Optional[tuple[Axis, Axis]]]:
+        if (fluxes := self.get_fluxes_df()) is None:
+            return None, None
+
+        fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(10, 4), sharey=True)
+        for k, v in self.model.stoichiometries_by_compounds[cpd].items():
+            if v < 0:
+                ax = ax2
+            else:
+                ax = ax1
+            (fluxes[k] * abs(v)).plot(ax=ax, label=k)
+
+        fig.suptitle(cpd)
+        ax1.set_title("Production")
+        ax2.set_title("Consumption")
+        ax1.legend(loc="upper left")
+        ax2.legend(loc="upper left")
+
+        return fig, (ax1, ax2)
