@@ -8,6 +8,7 @@ import copy
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Type, Union, cast
 
 from matplotlib.figure import Figure
+from typing_extensions import Self
 
 from ...typing import Array, ArrayLike, Axis
 from ...utils.plotting import plot, plot_grid
@@ -74,7 +75,7 @@ class _LinearLabelSimulate(_BaseSimulator[_LinearLabelModel]):
         v_ss: Dict[str, float],
         external_label: float = 1.0,
         test_run: bool = True,
-    ) -> None:
+    ) -> Self:
         self.model._y_ss = y_ss
         self.model._v_ss = v_ss
         self.model._external_label = external_label
@@ -88,6 +89,7 @@ class _LinearLabelSimulate(_BaseSimulator[_LinearLabelModel]):
 
         if test_run:
             self._test_run()
+        return self
 
     def get_label_position(self, compound: str, position: int) -> Optional[Array]:
         """Get relative concentration of a single isotopomer.
