@@ -75,7 +75,7 @@ class _Simulate(_BaseRateSimulator[Model]):
     ) -> dict[str, float]:
         par_names = list(p0.keys())
         x0 = list(p0.values())
-        p_orig = self.model.parameters.copy()
+        p_orig = self.model.get_parameters().copy()
 
         def residual(par_values: ArrayLike, data: ArrayLike) -> float:
             self.clear_results()
@@ -86,7 +86,7 @@ class _Simulate(_BaseRateSimulator[Model]):
             return cast(float, np.sqrt(np.mean(np.square(data - y_ss.flatten()))))
 
         res = dict(zip(par_names, minimize(residual, x0=x0, args=(data,)).x))
-        self.model.parameters = p_orig
+        self.model.update_parameters(p_orig)
         return res
 
     def fit_time_series(
@@ -97,7 +97,7 @@ class _Simulate(_BaseRateSimulator[Model]):
     ) -> dict[str, float]:
         par_names = list(p0.keys())
         x0 = list(p0.values())
-        p_orig = self.model.parameters.copy()
+        p_orig = self.model.get_parameters().copy()
         assert len(data) == len(time_points)
 
         def residual(
@@ -111,7 +111,7 @@ class _Simulate(_BaseRateSimulator[Model]):
             return cast(float, np.sqrt(np.mean(np.square(data - y))))
 
         res = dict(zip(par_names, minimize(residual, x0=x0, args=(data, time_points)).x))
-        self.model.parameters = p_orig
+        self.model.update_parameters(p_orig)
         return res
 
     def plot(
