@@ -22,7 +22,11 @@ __all__ = [
     "plot_flux_response_coefficients",
 ]
 import math
-from typing import Any, Dict, List, Optional, Tuple, Union, cast
+import sys
+import warnings
+from concurrent.futures import ProcessPoolExecutor
+from functools import partial
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, cast
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -81,6 +85,7 @@ def get_compound_elasticities_array(
     compounds: List[str],
     y: Dict[str, float],
     t: float = 0,
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
 ) -> Array:
@@ -108,6 +113,7 @@ def get_compound_elasticities_df(
     compounds: List[str],
     y: Dict[str, float],
     t: float = 0,
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
 ) -> pd.DataFrame:
@@ -131,6 +137,7 @@ def get_parameter_elasticity(
     parameter: str,
     y: Dict[str, float],
     t: float = 0,
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
 ) -> Array:
@@ -157,6 +164,7 @@ def get_parameter_elasticities_array(
     parameters: List[str],
     y: Dict[str, float],
     t: float = 0,
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
 ) -> Array:
@@ -184,6 +192,7 @@ def get_parameter_elasticities_df(
     parameters: List[str],
     y: Dict[str, float],
     t: float = 0,
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
 ) -> pd.DataFrame:
@@ -212,6 +221,7 @@ def _find_steady_state(
     y0: Union[ArrayLike, Dict[str, float]],
     tolerance: float,
     simulation_kwargs: Dict[str, Any] | None,
+    rel_norm: bool = False,
     **integrator_kwargs: Dict[str, Any],
 ) -> Tuple[Optional[Array], Optional[Array]]:
     """Simulate the system to steadt state."""
@@ -220,6 +230,7 @@ def _find_steady_state(
     t, y = s.simulate_to_steady_state(
         tolerance=tolerance,
         simulation_kwargs=simulation_kwargs,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )
     return t, y
@@ -229,10 +240,12 @@ def get_response_coefficients(
     model: Model,
     parameter: str,
     y: Union[Dict[str, float], Array, List],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     simulation_kwargs: Dict[str, Any] | None = None,
+    rel_norm: bool = False,
     **integrator_kwargs: Dict[str, Any],
 ) -> Tuple[Optional[Array], Optional[Array]]:
     """Get response of the steady state concentrations and fluxes to a change of the given parameter."""
@@ -245,6 +258,7 @@ def get_response_coefficients(
             y0=y,
             tolerance=tolerance,
             simulation_kwargs=simulation_kwargs,
+            rel_norm=rel_norm,
             **integrator_kwargs,
         )
         if t_ss is None or y_ss is None:
@@ -267,6 +281,7 @@ def get_response_coefficients(
             y0=y,
             tolerance=tolerance,
             simulation_kwargs=simulation_kwargs,
+            rel_norm=rel_norm,
             **integrator_kwargs,
         )
         if t_ss is None or y_ss is None:
@@ -291,37 +306,61 @@ def get_response_coefficients_array(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    multiprocessing: bool = True,
+    max_workers: int | None = None,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     **integrator_kwargs: Dict[str, Any],
 ) -> Tuple[Array, Array]:
     """Get response of the steady state concentrations and fluxes to a change of the given parameter."""
+    if sys.platform in ["win32", "cygwin"] and multiprocessing:
+        warnings.warn(
+            """
+                Windows does not behave well with multiple processes.
+                Falling back to threading routine."""
+        )
+
     crcs = np.full(
         shape=(len(parameters), len(model.get_all_compounds())), fill_value=np.nan
     )
     frcs = np.full(
         shape=(len(parameters), len(model.get_rate_names())), fill_value=np.nan
     )
-    for i, parameter in tqdm(
-        enumerate(parameters), total=len(parameters), disable=disable_tqdm
-    ):
-        crc, frc = get_response_coefficients(
-            model=model,
-            parameter=parameter,
-            y=y,
-            normalized=normalized,
-            displacement=displacement,
-            tolerance=tolerance,
-            simulation_kwargs=simulation_kwargs,
-            **integrator_kwargs,
+    _get_response_coefficients = partial(
+        get_response_coefficients,
+        model,
+        y=y,
+        normalized=normalized,
+        displacement=displacement,
+        tolerance=tolerance,
+        simulation_kwargs=simulation_kwargs,
+        rel_norm=rel_norm,
+        **integrator_kwargs,
+    )
+
+    rcs: Iterable[tuple[Array | None, Array | None]]
+    if sys.platform in ["win32", "cygwin"] or not multiprocessing:
+        rcs = tqdm(
+            map(_get_response_coefficients, parameters),
+            disable=disable_tqdm,
+            total=len(parameters),
         )
-        if crc is not None:
-            crcs[i] = crc
-        if frc is not None:
-            frcs[i] = frc
+    else:
+        with ProcessPoolExecutor(max_workers=max_workers) as pe:
+            rcs = tqdm(
+                pe.map(_get_response_coefficients, parameters),
+                disable=disable_tqdm,
+                total=len(parameters),
+            )
+
+    for i, (crc, frc) in enumerate(rcs):
+        crcs[i] = crc
+        frcs[i] = frc
     return crcs, frcs
 
 
@@ -329,10 +368,14 @@ def get_response_coefficients_df(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    multiprocessing: bool = True,
+    max_workers: int | None = None,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     **integrator_kwargs: Dict[str, Any],
 ) -> Tuple[pd.DataFrame, pd.DataFrame]:
@@ -346,6 +389,9 @@ def get_response_coefficients_df(
         tolerance=tolerance,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
+        multiprocessing=multiprocessing,
+        max_workers=max_workers,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )
     crcs_df = pd.DataFrame(
@@ -365,10 +411,12 @@ def get_concentration_response_coefficient(
     model: Model,
     parameter: str,
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     simulation_kwargs: Dict[str, Any] | None = None,
+    rel_norm: bool = False,
     **integrator_kwargs: Dict[str, Any],
 ) -> Optional[Array]:
     """Get response of the steady state concentrations to a change of the given parameter."""
@@ -380,6 +428,7 @@ def get_concentration_response_coefficient(
         displacement=displacement,
         tolerance=tolerance,
         simulation_kwargs=simulation_kwargs,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )[0]
 
@@ -388,10 +437,14 @@ def get_concentration_response_coefficients_array(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    multiprocessing: bool = True,
+    max_workers: int | None = None,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     **integrator_kwargs: Dict[str, Any],
 ) -> Array:
@@ -403,8 +456,11 @@ def get_concentration_response_coefficients_array(
         normalized=normalized,
         displacement=displacement,
         tolerance=tolerance,
+        multiprocessing=multiprocessing,
+        max_workers=max_workers,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )[0]
 
@@ -413,10 +469,14 @@ def get_concentration_response_coefficients_df(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     disable_tqdm: bool = False,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
+    multiprocessing: bool = True,
+    max_workers: int | None = None,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     **integrator_kwargs: Dict[str, Any],
 ) -> pd.DataFrame:
@@ -428,8 +488,11 @@ def get_concentration_response_coefficients_df(
         normalized=normalized,
         displacement=displacement,
         tolerance=tolerance,
+        multiprocessing=multiprocessing,
+        max_workers=max_workers,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )[0]
 
@@ -438,10 +501,12 @@ def get_flux_response_coefficient(
     model: Model,
     parameter: str,
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     simulation_kwargs: Dict[str, Any] | None = None,
+    rel_norm: bool = False,
     **integrator_kwargs: Dict[str, Any],
 ) -> Optional[Array]:
     """Get response of the steady state fluxes to a change of the given parameter."""
@@ -453,6 +518,7 @@ def get_flux_response_coefficient(
         displacement=displacement,
         tolerance=tolerance,
         simulation_kwargs=simulation_kwargs,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )[1]
 
@@ -461,10 +527,14 @@ def get_flux_response_coefficients_array(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    multiprocessing: bool = True,
+    max_workers: int | None = None,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     **integrator_kwargs: Dict[str, Any],
 ) -> Array:
@@ -476,8 +546,11 @@ def get_flux_response_coefficients_array(
         normalized=normalized,
         displacement=displacement,
         tolerance=tolerance,
+        multiprocessing=multiprocessing,
+        max_workers=max_workers,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )[1]
 
@@ -486,10 +559,14 @@ def get_flux_response_coefficients_df(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    multiprocessing: bool = True,
+    max_workers: int | None = None,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     **integrator_kwargs: Dict[str, Any],
 ) -> pd.DataFrame:
@@ -501,14 +578,18 @@ def get_flux_response_coefficients_df(
         normalized=normalized,
         displacement=displacement,
         tolerance=tolerance,
+        multiprocessing=multiprocessing,
+        max_workers=max_workers,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
+        rel_norm=rel_norm,
         **integrator_kwargs,
     )[1]
 
 
 def plot_coefficient_heatmap(
     df: pd.DataFrame,
+    *,
     title: str,
     cmap: str = "RdBu_r",
     rows: list[str] | None = None,
@@ -552,6 +633,7 @@ def plot_coefficient_heatmap(
 
 def plot_multiple(
     dfs: List[pd.DataFrame],
+    *,
     titles: List[str],
     cmap: str = "RdBu_r",
     annotate: bool = True,
@@ -608,10 +690,12 @@ def plot_concentration_response_coefficients(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     cmap: str = "RdBu_r",
     rows: list[str] | None = None,
@@ -642,7 +726,8 @@ def plot_concentration_response_coefficients(
         tolerance=tolerance,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
-        **integrator_kwargs,
+        rel_norm=rel_norm,
+        **integrator_kwargs,  # type: ignore
     )
     return plot_coefficient_heatmap(
         df=df,
@@ -665,10 +750,12 @@ def plot_flux_response_coefficients(
     model: Model,
     parameters: List[str],
     y: Dict[str, float],
+    *,
     normalized: bool = True,
     displacement: float = _DISPLACEMENT,
     tolerance: float = _DEFAULT_TOLERANCE,
     disable_tqdm: bool = False,
+    rel_norm: bool = False,
     simulation_kwargs: Dict[str, Any] | None = None,
     cmap: str = "RdBu_r",
     rows: list[str] | None = None,
@@ -699,7 +786,8 @@ def plot_flux_response_coefficients(
         tolerance=tolerance,
         simulation_kwargs=simulation_kwargs,
         disable_tqdm=disable_tqdm,
-        **integrator_kwargs,
+        rel_norm=rel_norm,
+        **integrator_kwargs,  # type: ignore
     )
     return plot_coefficient_heatmap(
         df=df,
