@@ -5,7 +5,6 @@ __all__ = [
 ]
 
 import copy
-
 import math  # noqa: F401, math needs to be imported for sbml functions
 import re
 import warnings
@@ -999,7 +998,7 @@ class Parser:
         # Initial assignments
         possible_sources: Dict[str, Any] = {
             **model.get_full_concentration_dict(y0),
-            **model.parameters,
+            **model.get_all_parameters(),
         }
         for par_name, assignment in self.converted_initial_assignments.items():
             py_func = self.create_python_function(func=assignment)
