@@ -368,7 +368,7 @@ def heatmap_from_dataframe(
         hm.update_scalarmappable()  # So that get_facecolor is an array
         xpos, ypos = np.meshgrid(np.arange(len(columns)), np.arange(len(rows)))
         for x, y, val, color in zip(
-            xpos.flat, ypos.flat, hm.get_array(), hm.get_facecolor()
+            xpos.flat, ypos.flat, hm.get_array().flat, hm.get_facecolor()
         ):
             text_kwargs["color"] = (
                 "black" if relative_luminance(color) > 0.45 else "white"
